@@ -1,5 +1,6 @@
 import json
 import os
+import textwrap
 import warnings
 from urllib.parse import urljoin
 
@@ -7,15 +8,24 @@ import flattentool
 import gspread
 import requests
 import sql
-from google.colab import auth, files
+from sqlalchemy.exc import ResourceClosedError
+
+try:
+    from google.colab import auth, files
+except ImportError:
+    # Assume we are in a testing environment.
+    from unittest.mock import Mock
+
+    files = Mock()
+    files.download.return_value = None
 from gspread_dataframe import set_with_dataframe
 from IPython import get_ipython
+from IPython.display import HTML
 from libcoveocds.config import LibCoveOCDSConfig
 from notebook import notebookapp
 from oauth2client.client import GoogleCredentials
-from pydrive.auth import GoogleAuth
-from pydrive.drive import GoogleDrive
-
+from pydrive2.auth import GoogleAuth
+from pydrive2.drive import GoogleDrive
 
 # Monkeypatch ipython-sql's sql run function, to add a comment linking to the
 # colab notebook that it's run from
@@ -26,7 +36,7 @@ old_run = sql.run.run
 
 def run(conn, _sql, *args, **kwargs):
     try:
-        comment = '/* https://colab.research.google.com/drive/{} */'.format(_notebook_id())
+        comment = f'/* https://colab.research.google.com/drive/{_notebook_id()} */'
     except KeyError:
         comment = "/* run from a notebook, but no colab id */"
     return old_run(conn, comment + _sql, *args, **kwargs)
@@ -89,9 +99,11 @@ def list_source_ids(pattern=''):
     Returns, as a ResultSet or DataFrame, a list of source IDs matching the given pattern.
 
     :param str pattern: a substring, like "paraguay"
-    :returns: the results as a pandas DataFrame or an ipython-sql `ResultSet <https://github.com/catherinedevlin/ipython-sql/blob/b24ac6e9410416eafde86ae22fd8d6f34acbe05d/src/sql/run.py#L99>`__, depending on whether ``%config SqlMagic.autopandas`` is ``True`` or ``False`` respectively. This is the same behaviour as ipython-sql's ``%sql`` magic.
+    :returns: the results as a pandas DataFrame or an ipython-sql :ipython-sql:`ResultSet<src/sql/run.py#L99>`,
+              depending on whether ``%config SqlMagic.autopandas`` is ``True`` or ``False`` respectively. This is the
+              same behaviour as ipython-sql's ``%sql`` magic.
     :rtype: pandas.DataFrame or sql.run.ResultSet
-    """  # noqa: E501
+    """
     sql = """
     SELECT source_id
     FROM collection
@@ -106,20 +118,20 @@ def list_source_ids(pattern=''):
     return get_ipython().magic(f'sql {sql}')
 
 
-def list_collections(source_id):
+def list_collections(source_id=None):
     """
     Returns, as a ResultSet or DataFrame, a list of collections with the given source ID.
 
     :param str source_id: a source ID
-    :returns: the results as a pandas DataFrame or an ipython-sql `ResultSet <https://github.com/catherinedevlin/ipython-sql/blob/b24ac6e9410416eafde86ae22fd8d6f34acbe05d/src/sql/run.py#L99>`__, depending on whether ``%config SqlMagic.autopandas`` is ``True`` or ``False`` respectively. This is the same behaviour as ipython-sql's ``%sql`` magic.
+    :returns: the results as a pandas DataFrame or an ipython-sql :ipython-sql:`ResultSet<src/sql/run.py#L99>`,
+              depending on whether ``%config SqlMagic.autopandas`` is ``True`` or ``False`` respectively. This is the
+              same behaviour as ipython-sql's ``%sql`` magic.
     :rtype: pandas.DataFrame or sql.run.ResultSet
-    """  # noqa: E501
-    sql = """
-    SELECT *
-    FROM collection
-    WHERE source_id = :source_id
-    ORDER BY id DESC
     """
+    sql = "SELECT * FROM collection"
+    if source_id:
+        sql += " WHERE source_id = :source_id"
+    sql += " ORDER BY id DESC"
 
     # This inspects locals to find source_id
     return get_ipython().magic(f'sql {sql}')
@@ -132,7 +144,11 @@ def set_search_path(schema_name):
 
     :param str schema_name: a schema name
     """
-    get_ipython().magic(f'sql SET search_path = {schema_name}, public')
+    try:
+        get_ipython().magic(f'sql SET search_path = {schema_name}, public')
+    # https://github.com/catherinedevlin/ipython-sql/issues/191
+    except ResourceClosedError:
+        pass
 
 
 def save_dataframe_to_sheet(dataframe, sheetname, prompt=True):
@@ -155,7 +171,7 @@ def save_dataframe_to_sheet(dataframe, sheetname, prompt=True):
         try:
             worksheet = sheet.add_worksheet(sheetname, dataframe.shape[0], dataframe.shape[1])
         except gspread.exceptions.APIError:
-            newsheetname = input('{} already exists, enter a new name:'.format(sheetname))
+            newsheetname = input(f'{sheetname} already exists, enter a new name:')
             worksheet = sheet.add_worksheet(newsheetname, dataframe.shape[0], dataframe.shape[1])
 
         set_with_dataframe(worksheet, dataframe)
@@ -171,7 +187,6 @@ def save_dataframe_to_spreadsheet(dataframe, name):
     """
     write_data_as_json(dataframe['release_package'][0], 'release_package.json')
 
-    # Use similar code to Toucan.
     config = LibCoveOCDSConfig().config
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore')  # flattentool uses UserWarning, so we can't set a specific category
@@ -188,8 +203,8 @@ def save_dataframe_to_spreadsheet(dataframe, name):
             output_format='xlsx',
         )
 
-    drive_file = _save_file_to_drive({'title': name + '.xlsx'}, 'flattened.xlsx')
-    print('Uploaded file with ID {!r}'.format(drive_file['id']))
+    drive_file = _save_file_to_drive({'title': f'{name}.xlsx'}, 'flattened.xlsx')
+    print(f"Uploaded file with ID {drive_file['id']!r}")
 
 
 def download_dataframe_as_csv(dataframe, filename):
@@ -221,9 +236,9 @@ def get_ipython_sql_resultset_from_query(sql):
     Parameters are taken from the scope this function is called from (same behaviour as ipython-sql's ``%sql`` magic).
 
     :param str sql: a SQL statement
-    :returns: the results as a `ResultSet <https://github.com/catherinedevlin/ipython-sql/blob/b24ac6e9410416eafde86ae22fd8d6f34acbe05d/src/sql/run.py#L99>`__
+    :returns: the results as a :ipython-sql:`ResultSet<src/sql/run.py#L99>`
     :rtype: sql.run.ResultSet
-    """  # noqa: E501
+    """
     ipython = get_ipython()
     autopandas = ipython.magic('config SqlMagic.autopandas')
     # Disable autopandas, so we know that the sql magic call will always return
@@ -260,7 +275,7 @@ def download_package_from_query(sql, package_type=None):
         package = {'releases': data}
 
     package.update(package_metadata)
-    download_data_as_json(package, '{}_package.json'.format(package_type))
+    download_data_as_json(package, f'{package_type}_package.json')
 
 
 def download_package_from_ocid(collection_id, ocid, package_type):
@@ -304,7 +319,7 @@ def download_package_from_ocid(collection_id, ocid, package_type):
         package = {'releases': data}
 
     package.update(package_metadata)
-    download_data_as_json(package, '{}_{}_package.json'.format(ocid, package_type))
+    download_data_as_json(package, f'{ocid}_{package_type}_package.json')
 
 
 def write_data_as_json(data, filename):
@@ -331,6 +346,192 @@ def _save_file_to_drive(metadata, filename):
     return drive_file
 
 
+def render_json(json_string):
+    """
+    Renders JSON into collapsible HTML.
+
+    :param json_string: JSON-deserializable string
+    """
+    if not isinstance(json_string, str):
+        json_string = json.dumps(json_string)
+    return HTML(f"""
+        <script
+        src="https://cdn.jsdelivr.net/gh/caldwell/renderjson@master/renderjson.js">
+        </script>
+        <script>
+        renderjson.set_show_to_level(1)
+        document.body.appendChild(renderjson({json_string}))
+        new ResizeObserver(google.colab.output.resizeIframeToContent).observe(document.body)
+        </script>
+        """)
+
+
+def calculate_coverage(fields, scope=None, sql=True, sql_only=False):
+    """
+    Calculates the coverage of one or more fields using the summary tables produced by Kingfisher Summarize's
+    `--field-lists` option. Returns the coverage of each field and the co-occurrence coverage of all the fields.
+
+    `scope` is the Kingfisher Summarize table to measure coverage against, e.g. `awards_summary`.
+    The number of rows in this table is used as the denominator when calculating the coverage.
+
+    `fields` is a list of fields to measure the coverage of, specified using JSON Pointer.
+
+    To specify fields that are children of the scope table, you can use either an absolute pointer or a relative
+    pointer prefixed with `:`, e.g. if `scope` is set to 'awards_summary', then `awards/value/amount` and
+    `:value/amount` refer to the same field. Coverage of such fields is measured against the number of rows
+    in the `scope` table.
+
+    To specify fields that are not children of the scope table, use an absolute path,
+    e.g. `tender/procurementMethod`. Coverage of such fields is measured against the number of releases/records.
+
+    For arrays, a field is counted if it appears in **any** object in the array,
+    e.g. if `scope` is set to `awards_summary` and `field` is set to `:items/description`,
+    at least one item must have a description for the coverage to be non-zero.
+
+    To specify that a field must appear in **all** objects in the array, prepend the field with `ALL `,
+    e.g. if `scope` is set to `awards_summary` and `field` is set to `ALL :items/description`,
+    all items must have a description for the coverage to be non-zero.
+
+    If `scope` is set to `awards_summary`, specify fields on related contracts by prefixing the path with
+    `:contracts/`, e.g. to measure how many awards have a value and a related contract with a period, set `scope`
+    to `awards` and `fields` to `[':value', ':contracts/period']`. Similarly, if `scope` is set to
+    `contracts_summary`, specify fields on related awards by prefixing the path with `:awards/`.
+
+
+    :param list fields: a list of fields as described above.
+    :param str scope: table name as described above; defaults to the parent table of the first item in the fields list.
+    :param bool sql: print the SQL query generated by the function.
+
+    :returns: the coverage of each field and the co-occurrence coverage of all the fields as a pandas DataFrame or an
+                  ipython-sql :ipython-sql:`ResultSet<src/sql/run.py#L99>`, depending on whether
+                  ``%config SqlMagic.autopandas`` is ``True`` or ``False`` respectively. This is the same behaviour as
+                  ipython-sql's ``%sql`` magic.
+    :rtype: pandas.DataFrame or sql.run.ResultSet
+    """
+
+    def get_all_tables():
+        views = get_ipython_sql_resultset_from_query(
+            "SELECT viewname FROM pg_catalog.pg_views WHERE schemaname = ANY (CURRENT_SCHEMAS(false))"
+        )
+        tables = get_ipython_sql_resultset_from_query(
+            "SELECT tablename FROM pg_catalog.pg_tables WHERE schemaname = ANY (CURRENT_SCHEMAS(false))"
+        )
+        return [x[0] for x in list(views) + list(tables)]
+
+    def get_table_and_path(field, scope_table):
+
+        if field.startswith(':'):
+            return scope_table, field[1:]
+
+        path = field.split("/")
+        table_candidates = [
+            "_".join(path[:-i]) for i in reversed(range(1, len(path)))
+        ]
+        table = "release_summary"
+
+        for num, table_candidate in enumerate(table_candidates):
+            if scope_table[:-8] == table_candidate:  # remove "_summary" from `scope_table`
+                path = path[num+1:]
+                table = scope_table
+                break
+        return table, "/".join(path)
+
+    def get_scope_table(field):
+
+        all_tables = get_all_tables()
+        path = field.split("/")
+        table_candidates = {
+            "_".join(path[:-i]) for i in range(1, len(path))
+        }
+        table = "release"
+
+        for table_candidate in table_candidates:
+            if f"{table_candidate}_summary" in all_tables:
+                table = table_candidate
+        return f"{table}_summary"
+
+    def coverage_wrapper(condition, field):
+        field_name = field.replace("/", "_").replace(" ", "_").lower()
+        return f"ROUND(SUM(CASE WHEN {condition} THEN 1 ELSE 0 END) * 100.0 / count(*), 2) AS {field_name}_percentage"
+
+    def any_condition(field, current_scope_table):
+        return f"{current_scope_table}.field_list ? '{field}'"
+
+    def all_condition(field, current_scope_table):
+        split_field = field.split("/")
+        one_to_manys = [field for field in split_field[:-1] if field.endswith("s")]
+
+        if not one_to_manys:
+            nearest_parent_one_to_many = split_field[0]
+        else:
+            nearest_parent_one_to_many = one_to_manys[-1]
+
+        if len(one_to_manys) > 1:
+            print(
+                f"""WARNING: The results of this query might be inacurate, you will need to check that
+                 `{', '.join(one_to_manys[:-1])}` fields are one to one with `{current_scope_table[:-8]}`
+                 and that `{', '.join(one_to_manys[:-1])}` exists for all `{current_scope_table[:-8]}` """
+            )
+
+        return f"""coalesce({current_scope_table}.field_list ->> '{field}' =
+                  {current_scope_table}.field_list ->> '{nearest_parent_one_to_many}', false)"""
+
+    def release_summary_join(scope_table, join_to_release):
+        if not join_to_release:
+            return ""
+        return f"""JOIN
+        release_summary ON release_summary.id = {scope_table}.id"""
+
+    if not scope:
+        field = fields[0].split()[-1]
+        scope = get_scope_table(field)
+
+    scope_table = scope
+
+    join_to_release = False
+
+    conditions = []
+
+    query_parts = []
+
+    for field in fields:
+        split_field = field.split()
+        field_name = split_field[-1]
+
+        table, path = get_table_and_path(field_name, scope)
+
+        if table == "release_summary" and scope_table != "release_summary":
+            join_to_release = True
+
+        if len(split_field) == 2 and split_field[0].lower() == "all":
+            condition = all_condition(path, table)
+        else:
+            condition = any_condition(path, table)
+
+        conditions.append(condition)
+        query_parts.append(coverage_wrapper(condition, path))
+
+    query_parts.append(
+        coverage_wrapper(" AND \n              ".join(conditions), "total")
+    )
+
+    select = ",\n            ".join(query_parts)
+    select = textwrap.dedent(f"""
+        SELECT
+            count(*) AS total_{scope},
+            {select}
+        FROM
+        {scope_table}
+        {release_summary_join(scope_table, join_to_release)}
+    """)
+
+    if sql:
+        print(select)
+    if sql_only:
+        return select
+    return get_ipython().run_cell_magic("sql", "", select)
+
+
 class OCDSKingfisherColabError(Exception):
     """Base class for exceptions from within this package"""
 
