@@ -6,7 +6,6 @@ import json
 import math
 import os
 import textwrap
-from io import StringIO
 from unittest.mock import patch
 from zipfile import ZipFile
 
@@ -14,9 +13,10 @@ import pandas
 import pytest
 from IPython import get_ipython
 
-from ocdskingfishercolab import (UnknownPackageTypeError, download_dataframe_as_csv, download_package_from_ocid,
-                                 download_package_from_query, get_ipython_sql_resultset_from_query, list_collections,
-                                 list_source_ids, save_dataframe_to_spreadsheet, set_search_path)
+from ocdskingfishercolab import (UnknownPackageTypeError, calculate_coverage, download_dataframe_as_csv,
+                                 download_package_from_ocid, download_package_from_query,
+                                 get_ipython_sql_resultset_from_query, list_collections, list_source_ids,
+                                 save_dataframe_to_spreadsheet, set_search_path)
 
 
 def path(filename):
@@ -48,7 +48,7 @@ def test_set_search_path(db):
     get_ipython().magic('sql show search_path')['search_path'][0] == 'test, public'
 
 
-@patch('google.colab.files.download')
+@patch('ocdskingfishercolab.files.download')
 def test_download_dataframe_as_csv(download, tmpdir):
     d = {'col1': [1, 2], 'col2': [3, 4]}
     df = pandas.DataFrame(data=d)
@@ -62,7 +62,7 @@ def test_download_dataframe_as_csv(download, tmpdir):
         assert data == ',col1,col2\n0,1,3\n1,2,4\n'
 
 
-@patch('google.colab.files.download')
+@patch('ocdskingfishercolab.files.download')
 @patch('ocdskingfishercolab._notebook_id', _notebook_id)
 def test_download_package_from_ocid_release(download, db, tmpdir):
     with chdir(tmpdir):
@@ -85,7 +85,7 @@ def test_download_package_from_ocid_release(download, db, tmpdir):
         download.assert_called_once_with('ocds-213czf-1_release_package.json')
 
 
-@patch('google.colab.files.download')
+@patch('ocdskingfishercolab.files.download')
 @patch('ocdskingfishercolab._notebook_id', _notebook_id)
 def test_download_package_from_ocid_record(download, db, tmpdir):
     with chdir(tmpdir):
@@ -111,7 +111,7 @@ def test_download_package_from_ocid_record(download, db, tmpdir):
         download.assert_called_once_with('ocds-213czf-1_record_package.json')
 
 
-@patch('google.colab.files.download')
+@patch('ocdskingfishercolab.files.download')
 @patch('ocdskingfishercolab._notebook_id', _notebook_id)
 def test_download_package_from_ocid_path_separator(download, db, tmpdir):
     with chdir(tmpdir):
@@ -138,7 +138,7 @@ def test_download_package_from_ocid_other():
     assert str(excinfo.value) == "package_type argument must be either 'release' or 'record'"
 
 
-@patch('google.colab.files.download')
+@patch('ocdskingfishercolab.files.download')
 @patch('ocdskingfishercolab._notebook_id', _notebook_id)
 def test_download_package_from_query_release(download, db, tmpdir):
 
@@ -168,7 +168,7 @@ def test_download_package_from_query_release(download, db, tmpdir):
         download.assert_called_once_with('release_package.json')
 
 
-@patch('google.colab.files.download')
+@patch('ocdskingfishercolab.files.download')
 @patch('ocdskingfishercolab._notebook_id', _notebook_id)
 def test_download_package_from_query_record(download, db, tmpdir):
 
@@ -280,9 +280,8 @@ def test_list_collections(db):
     assert math.isnan(actual['transform_from_collection_id'][2])
 
 
-@patch('sys.stdout', new_callable=StringIO)
 @patch('ocdskingfishercolab._save_file_to_drive')
-def test_save_dataframe_to_spreadsheet(save, stdout, tmpdir):
+def test_save_dataframe_to_spreadsheet(save, capsys, tmpdir):
     save.return_value = {'id': 'test'}
 
     d = {'release_package': [{'releases': [{'ocid': 'ocds-213czf-1'}]}]}
@@ -302,6 +301,23 @@ def test_save_dataframe_to_spreadsheet(save, stdout, tmpdir):
             assert _worksheets_length(actual) == _worksheets_length(expected) == 1
             assert actual.read('xl/worksheets/sheet1.xml') == expected.read('xl/worksheets/sheet1.xml')
 
-        assert stdout.getvalue() == "Uploaded file with ID 'test'\n"
+        assert capsys.readouterr().out == "Uploaded file with ID 'test'\n"
 
         save.assert_called_once_with({'title': 'yet_another_excel_file.xlsx'}, 'flattened.xlsx')
+
+
+@patch('ocdskingfishercolab._notebook_id', _notebook_id)
+def test_calculate_coverage(db, tmpdir):
+
+    sql = calculate_coverage(["ocid"], scope="release_summary", sql_only=True)
+
+    # only seperated to reduce line length
+    case_statement = "CASE WHEN release_summary.field_list ? 'ocid' THEN 1 ELSE 0 END"
+
+    assert sql.strip() == textwrap.dedent(f'''
+    SELECT
+        count(*) AS total_release_summary,
+        ROUND(SUM({case_statement}) * 100.0 / count(*), 2) AS ocid_percentage,
+        ROUND(SUM({case_statement}) * 100.0 / count(*), 2) AS total_percentage
+    FROM
+    release_summary''').strip()
